/**************************************************************** 
 * Description: 二进制分解的 Trie,同时满足二叉查找树，线段树和平衡树性质，具有高可扩展性
 * Author: Alex Li
 * Date: 2024-08-10 16:56:37
 * LastEditTime: 2024-08-11 16:24:51
****************************************************************/
#include <iostream>
#include <cstdio>
#include <algorithm>
using namespace std;

const int maxlog = 25;
const int MAXN = 100010;
const int num = 10000000;

int id = 1;   //id=1是根结点，默认有根结点。
// 存储节点的子节点ch[id][0] 表示节点 id 的左子节点的编号。 ch[id][1] 表示节点 id 的右子节点的编号。
int ch[MAXN * maxlog][2];  
int sz[MAXN * maxlog];  // 存储以该节点为根的子树的大小

// 创建一个新节点，并初始化该节点的子节点和大小
int newnode() {
    id++; 
    ch[id][0] = ch[id][1] = sz[id] = 0;
    return id;
}

// 插入或删除一个数字 x
void ins(int x, int d) {
    int u = 1;  // 从根节点开始
    for (int i = maxlog - 1; i >= 0; i--) {
        int v = (x >> i) & 1;  // 获取第 i 位的值（从最高位开始）
        if (!ch[u][v]) ch[u][v] = newnode();  // 如果该子节点不存在，则创建
        u = ch[u][v];  // 继续向下遍历
        sz[u] += d;  // 更新子树的大小
    }
}

// 查找第 k小的数字
int kth(int k) {
    int u = 1;
    int x = 0;
    for (int i = maxlog - 1; i >= 0; i--) {
        if (sz[ch[u][0]] >= k) {
            u = ch[u][0];  // 移动到左子树
        } else {
            x |= (1 << i);  // 将当前位设为 1
            k -= sz[ch[u][0]];  // 减去左子树的大小
            u = ch[u][1];  // 移动到右子树
        }
    }
    return x;
}

// 查找小于 x 的数字的数量
int nlt(int x) {
    int ans = 0;
    int u = 1;
    for (int i = maxlog - 1; i >= 0; i--) {
        if ((x >> i) & 1) {
            ans += sz[ch[u][0]];  // 累加左子树的大小
            u = ch[u][1];  // 移动到右子树
        } else {
            u = ch[u][0];  // 移动到左子树
        }
        if (!u) break;  // 如果当前节点为空，退出循环
    }
    return ans;
}


// 查找 x 的前驱（小于 x 的最大值）
int pre(int x) {
    int ans;
    ans = kth(nlt(x));  // 查找 x 的前驱
    return ans;
}

// 查找 x 的后继（大于 x 的最小值）
int next(int x) {
    int ans;
    ans = kth(nlt(x + 1) + 1);  // 查找 x 的后继
    return ans;
}

int main() {
    int n;
    cin >> n;
    for (int i = 0; i < n; i++) {
        int ord, t;
        cin >> ord >> t;
        switch (ord) {
            case 1:
                ins(t + num, 1);  // 插入 t
                break;
            case 2:
                ins(t + num, -1);  // 删除 t
                break;
            case 3:
                cout << nlt(t + num) + 1 << endl;  // 查询 t 的排名
                break;
            case 4:
                cout << kth(t) - num << endl;  // 查询排名为 t 的数
                break;
            case 5:
                cout << pre(t + num) - num << endl;  // 查询 t 的前驱
                break;
            case 6:
                cout << next(t + num) - num << endl;  // 查询 t 的后继
                break;
        }
    }
    return 0;
}
